xquery version "3.1";
(:~
: This module prepares the index html.
: @author Mathias Göbel
: @version 1.0
: @see https://fontane-nb.dariah.eu/register.html
:)
module namespace f-indexviewer="http://fontane-nb.dariah.eu/indexviewer";

import module namespace f-indexproc="http://fontane-nb.dariah.eu/indexproc" at "index-processor.xqm";
import module namespace config="http://textgrid.de/ns/SADE/config" at "../config/config.xqm";

declare namespace ore="http://www.openarchives.org/ore/terms/";
declare namespace rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace test="http://exist-db.org/xquery/xqsuite";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

declare
  %test:setUp
function f-indexviewer:setup() {
if( xmldb:collection-available("/db/sade-projects/textgrid") )
then true()
else
    (: read test config :)
    let $testconfig := doc("tests.xml")//testdata[@module="f-indexviewer"]
    (: prepare the collections :)
    let $prepare-collections :=
        for $create in $testconfig//create/collection
        return
            xmldb:create-collection($create/@target, $create/@new)
    (: prepare documents :)
    let $prepare-documents :=
        for $create in $testconfig//create/document
        return
            xmldb:store($create/@collection, $create/@name, $create/*)
    let $prepare-documents-from-template :=
        for $create in $testconfig//create/document-template
        let $template := $create/template/*
        return
            for $data in $create//data
            let $doc := doc( xmldb:store($create/@collection, $data/@name, $template) )
            return
                update insert $data/* into $doc//tei:body
    return true()
};
declare
  %test:tearDown
function f-indexviewer:teardown(){
    xmldb:remove("/db/sade-projects")
};

(: Main cache function
 : @return path to the cached document :)
declare
  %test:name("cache")

  %test:assertTrue
(:  %test:pending (: throws NPE on non-initialized db. needs some data from the grid. :):)
function local:cache()
 {
let $set-option := util:declare-option("exist:output-size-limit", "2000000")
let $collection-uri := "/db/sade-projects/textgrid/data/xml/xhtml",
    $resource := "index.xml"
let $agg := '253st.xml' (: TextGrid Aggregation containing all index files :)
let $uris := (doc('/db/sade-projects/textgrid/data/xml/agg/' || $agg)
          //ore:aggregates/substring-after(@rdf:resource, 'textgrid:')
        )[. != "25547"] (: omit the bibliography here :)
let $lastMod := $uris ! xmldb:last-modified($f-indexproc:dataPath, . || '.xml')

return
    if(not(local:login($collection-uri)))
    then error(QName("FONTANE", "INDEXVIEW1"), "unable to authenticate.")
    else
        if( xmldb:last-modified($collection-uri, $resource) gt max($lastMod) )
        then
            (: prerendered index younger than index files: do nothing :)
            $collection-uri || "/" || $resource
        else
            (: create the cache :)
            xmldb:store($collection-uri, $resource, <xhtml:div>{util:eval('f-indexproc:register()')}</xhtml:div>)
};

(: forces the creation of a new cache file
 : @return document-node() of the index cache file :)
declare function f-indexviewer:prepare-new-cache()
as document-node() {
let $path := "/db/sade-projects/textgrid/data/xml/xhtml/index.xml"
let $cleanup :=
    if( doc-available($path) )
    then xmldb:remove("/db/sade-projects/textgrid/data/xml/xhtml", "index.xml")
    else true()
return
    doc( local:cache() )
};

(:~
 : Recursive transformation of the index document, if a specific item is requested.
 : @param $nodes – node to be parsed
 : @param $id – a single id prepared with generate-id()
 : :)
declare function local:transform-entity($nodes as node()+, $id as xs:string)
as node()* {
for $node in $nodes
let $thisId := generate-id($node)
return
    if ($thisId = $id)
    then
      (: set 'active' class :)
        element { node-name($node) } {
            $node/@*[local-name() != "class"],
            attribute class {
                string($node/@class),
                "fhighlighted"
            },
            local:transform-entity($node/node(), $id)
        }
    else if(starts-with($id, $thisId))
    then
        typeswitch ($node)
        case element(*) return
            element { node-name($node) } {
                if(contains($node/@class, "tab-pane"))
                then
                    ($node/@*[local-name() != "class"],
                    attribute class {
                        string($node/@class),
                        "active"
                    })
                else
                    $node/@*,
                local:transform-entity($node/node(), $id)
            }
        default return
            $node
    else $node
};

declare function f-indexviewer:main($model as map()*, $node as node(), $index as xs:string) {
if ($f-indexproc:notebookParam)
then
    f-indexproc:register($index)[./@id = $index]
    (: xhtml prefixes w/o xmlns attr in the output. work around with: :)
    => serialize() => parse-xml()
else
    let $doc := doc( local:cache() )
    return
        if ($f-indexproc:getEntity)
        then
            let $id := generate-id($doc//xhtml:li[@id = $f-indexproc:getEntity])
            return
                local:transform-entity($doc/xhtml:div/xhtml:div[@id = $index], $id)
        else
            $doc/*/xhtml:div[@id = $index]
};

declare function f-indexviewer:tabs($nodes as node(), $model as map(*), $index as xs:string) {
    local:transfrom-tabs($nodes/*, $index)
};

(:~
 : Recursive transformation of a given node from the template to a
 : represent a list of tabs with current index view marked `active`.
 : @param $nodes – nodes to transform
 : @param $index – the index that should be marked `active`
 :   :)
declare
  %test:name("mark active tab")

  %test:arg("nodes", "<li xmlns=""http://www.w3.org/1999/xhtml""><a href=""register-listEvent.html"">Ereignisse</a></li>")
  %test:arg("index", "listEvent")
  %test:assertXPath("$result/@class = 'active'")

  %test:arg("nodes", "<div xmlns=""http://www.w3.org/1999/xhtml""/>")
  %test:arg("index", "listEvent")
  %test:assertEmpty

function local:transfrom-tabs($nodes as element()+, $index as xs:string) {
for $node in $nodes
return
    typeswitch ( $node )
    case element(xhtml:div) return
        $node/node() ! local:transfrom-tabs(., $index)
    case element(xhtml:ul) return
        element xhtml:ul {
            $node/@*,
            $node/* ! local:transfrom-tabs(., $index)
        }
    case element(xhtml:li) return
        element xhtml:li {
            $node/@*[not(@class)],
            if($node/xhtml:a/@href = "register-" || $index || ".html")
            then
                attribute class { $node/@class ! string(.), "active"}
            else $node/@class,
            $node/* ! local:transfrom-tabs(., $index)
        }
    case element(xhtml:a) return
        element xhtml:a {
            if($f-indexproc:notebookParam)
            then
                ($node/@*[local-name() != "href"],
                attribute href { string($node/@href) || "?nb=" || $f-indexproc:notebookParam })
            else $node/@*,
            $node/(*,text())
        }
    default return
        $node
};

declare function local:login($col) {
(: check for db availability :)
if(httpclient:get(xs:anyURI("http://localhost:8080"), false(), ())/@statusCode = 200)
then
    if(false() = xmldb:login($col, config:get('sade.user'), config:get("sade.password")))
    then (xmldb:login($col, "admin", ""))
    else true()
else true()
};
